#ifndef art_Framework_Principal_RunStopwatch_h
#define art_Framework_Principal_RunStopwatch_h

/*----------------------------------------------------------------------

Simple "guard" class as suggested by Chris Jones to start/stop the
Stopwatch: creating an object of type RunStopwatch starts the clock
pointed to, deleting it (when it goes out of scope) automatically
calls the destructor which stops the clock.

----------------------------------------------------------------------*/


#include "cetlib/cpu_timer.h"
#include <memory>

namespace art
{
  class RunStopwatch {

  public:
    typedef std::shared_ptr<cet::cpu_timer> StopwatchPointer;

    RunStopwatch(const StopwatchPointer& ptr): stopwatch_(ptr) {
      stopwatch_->start();
    }

    ~RunStopwatch() {
      stopwatch_->stop();
    }

  private:
    StopwatchPointer stopwatch_;
  };
}
#endif /* art_Framework_Principal_RunStopwatch_h */

// Local Variables:
// mode: c++
// End:
